
// TrampolineClient.java
// Andrew Davison, March 2009, ad@fivedots.coe.psu.ac.th

/* An example of Scratch's remote sensor connections.

   A Scratch/Java application where Java controls and updates
   a Scratch trampoline game (JavaTrampoline.sb).

   TrampolineClient is mostly a GUI with numerous buttons:

		* start/stop buttons for the scratch application
        * a text area to show messages sent to the client from the application
        * a clear button to wipe the text area
        * up/down/left/right navigation buttons to control the trampoline artist
        * a combo box with a selection of sounds for the artist
        * a sound button to make the artist output a sound

  Clicking the close box of the window, causes the client to
  disconnect from scratch

  There is a separate threaded TrampolineWatcher object for processing 
  messages coming from Scratch.

  Most of the Scratch interface code is located in actionPerformed(),
  and a bit in the construictor and closeDown().

  Before this Java program is started, the Scratch program must 
  already be loaded, and remote sensor connections switched on.
*/

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;



public class TrampolineClient extends JFrame implements ActionListener
{
  private static final String[] SPRITE_SOUNDS = { "Laugh-female", "meow", "Laser1" };
        // these names should be the same as the sounds used in the Scratch application

  private ScratchIO scratchIO = null;
  private TrampolineWatcher watcher;

  private JTextArea jtaMesgs;                  // output text area
  private JButton jbStart, jbStop, jbClear;    // start/stop/clear buttons
  private JButton jbLeft, jbUp, jbRight, jbDown, jbSound;    // navigation and sound buttons
  private JComboBox soundsList;


  public TrampolineClient()
  {
     super("Trampoline Client");
     scratchIO = new ScratchIO();

     initializeGUI();
     watcher = new TrampolineWatcher(this, scratchIO);
     watcher.start();    // start watching for scratch msgs

     addWindowListener( new WindowAdapter() {
       public void windowClosing(WindowEvent e)
       { closeDown(); }
     });

     setSize(220,450);
     setResizable(false);

     positionOnRight();
     setVisible(true);
  } // end of TrampolineClient();


  private void initializeGUI()
  // Text area in center, and start/stop and navigation buttons below
  {
    Container c = getContentPane();
    c.setLayout( new BorderLayout() );

    // message display area
    jtaMesgs = new JTextArea(7, 7);
    jtaMesgs.setEditable(false);
    JScrollPane jsp = new JScrollPane(jtaMesgs);
    c.add( jsp, "Center");

    JPanel ctrlPanel = new JPanel();  // a column of controls
    ctrlPanel.setLayout( new BoxLayout(ctrlPanel, BoxLayout.Y_AXIS));
    ctrlPanel.add( makeExecPanel());
    ctrlPanel.add( selectSoundPanel());
    ctrlPanel.add( makeNavigationPanel());
    c.add( ctrlPanel, "South");
  }  // end of initializeGUI()



  private JPanel makeExecPanel()
  // start and stop buttons
  {
    JPanel execPanel = new JPanel();

    jbStart = new JButton("Start");
    jbStart.addActionListener(this);
    execPanel.add(jbStart);

    jbStop = new JButton("Stop");
    jbStop.addActionListener(this);
    execPanel.add(jbStop);

    jbClear = new JButton("Clear");
    jbClear.addActionListener(this);
    execPanel.add(jbClear);

    return execPanel;
  }  // end of makeExecPanel()



  private JPanel selectSoundPanel()
  // select the sound used by the Scratch sprite
  {
    JPanel soundPanel = new JPanel();

    soundPanel.add( new JLabel("Sprite sound: "));

    soundsList = new JComboBox(SPRITE_SOUNDS);
    soundsList.setSelectedIndex(0);   // laugh-female
    soundsList.addActionListener(this);
    soundPanel.add(soundsList);

    scratchIO.updateMsg("soundName", SPRITE_SOUNDS[0]);

    return soundPanel;
  }  // end of selectSoundPanel()



  private JPanel makeNavigationPanel()
  // navigation and sound buttons, in a 3x3 grid
  {
    JPanel p1 = new JPanel( new GridLayout(3,3) );
    p1.setOpaque(false);

    // top row
    p1.add(Box.createRigidArea(new Dimension(48,48)));

    jbUp = makeButton("up");
    p1.add(jbUp);

    p1.add(Box.createRigidArea(new Dimension(48,48)));

    // middle row
    jbLeft = makeButton("left");
    p1.add(jbLeft);

    jbSound = makeButton("speak");
    p1.add(jbSound);

    jbRight = makeButton("right");
    p1.add(jbRight);

    // bottom row
    p1.add(Box.createRigidArea(new Dimension(48,48)));

    jbDown = makeButton("down");
    p1.add(jbDown);

    p1.add(Box.createRigidArea(new Dimension(48,48)));

    return p1;
  }  // end of makeNavigationPanel()



  private JButton makeButton(String iconFnm)
  // make a button from an image icon loaded from the icons/ dir
  {
    ImageIcon im =  new ImageIcon("icons/" + iconFnm + ".png");

    // don't draw the background of the button, or any text
    JButton jb = new JButton(im);
    jb.setContentAreaFilled(false);
    jb.setBorderPainted(false);
    jb.setBorder(null);
    jb.setIconTextGap(0);
    jb.setText(null);

    // highlight using another version of the icon
    ImageIcon imPressed =  new ImageIcon("icons/" + iconFnm + "Pressed.png");
    jb.setPressedIcon(imPressed);
    jb.setFocusPainted(false);

    // fix button size
    jb.setMargin(new Insets(0, 0, 0, 0));
    int h = im.getIconHeight();
    int w = im.getIconWidth();
    jb.setPreferredSize(new Dimension(w,h));
    jb.setMaximumSize(new java.awt.Dimension(w,h));
    jb.setMinimumSize(new java.awt.Dimension(w,h));

    jb.addActionListener(this);
    return jb;
  }  // end of makeButton()



  private void closeDown()
  // stop the watcher and then the client
  {
    watcher.finish();
    scratchIO.broadcastMsg("stopEvent");
    scratchIO.closeDown();
    System.exit(0); 
  } // end of closeDown()



  private void positionOnRight()
  // position this frame over on the rhs of the screen
  {
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();  
    int x = (screenSize.width - getWidth()) - 40;  
    int y = (screenSize.height - getHeight())/2;  
    setLocation(x, y);  
  }  // end of positionOnRight()
    


   public void actionPerformed(ActionEvent e)
   /* A button has been pressed; one of:
          jbStart, jbStop, jbLeft, jbUp, jbRight, jbDown, jbSound
      or a sound names combo box item has been selected.
   */
   {
     // exec buttons
     if (e.getSource() == jbStart)
       scratchIO.broadcastMsg("startEvent");
     else if (e.getSource() == jbStop)
       scratchIO.broadcastMsg("stopEvent");
     else if (e.getSource() == jbClear)
       jtaMesgs.setText(null);

     // navigation buttons
     else if (e.getSource() == jbLeft)
       scratchIO.broadcastMsg("leftEvent"); 
     else if (e.getSource() == jbUp)
       scratchIO.broadcastMsg("upEvent");
     else if (e.getSource() == jbRight)
       scratchIO.broadcastMsg("rightEvent");
     else if (e.getSource() == jbDown)
       scratchIO.broadcastMsg("downEvent");
     else if (e.getSource() == jbSound)
       scratchIO.broadcastMsg("soundEvent");

     // sound selector
     else if (e.getSource() == soundsList) {
       String soundName = (String)soundsList.getSelectedItem();
       System.out.println("Selected: " + soundName);
       scratchIO.updateMsg("soundName", soundName);
     }
     else
       System.out.println("Unknown GUI action: " + e);
   }  // end of actionPerformed()




  public void showMsg(final String msg)
  /* We're updating the messages text area, so the code should
     be carried out by Swing's event dispatching thread, which is 
     achieved by calling invokeLater(). 

     msg must be final to be used inside the inner class for Runnable.
  */
  { 
    // System.out.println("showMsg(): " + msg);
    Runnable updateMsgsText = new Runnable() {
      public void run() 
      { jtaMesgs.append(msg);  // append message to text area
        jtaMesgs.setCaretPosition( jtaMesgs.getText().length() );
            // move insertion point to the end of the text
      }
    };
    SwingUtilities.invokeLater( updateMsgsText );
  } // end of showMsg()


  // ------------------------------------

  public static void main(String args[]) 
  {  new TrampolineClient();  }

} // end of TrampolineClient class

